import json
from os.path import exists, expanduser, join

setting_file = join(expanduser("~"), ".meicanrc")


class MeiCanSetting(object):
    def __init__(self):
        self._settings = {}
        if not exists(setting_file):
            return
        with open(setting_file, encoding="utf-8") as f:
            self._settings = json.load(f)

    def save(self):
        with open(setting_file, str("w"), encoding="utf-8") as f:
            f.write(json.dumps(self._settings, ensure_ascii=False, indent=2))

    def load_credentials(self):
        for key in ["username", "password"]:
            if key not in self._settings:
                self._settings[key] = input("please input your meican {}: ".format(key))
        self.save()

    def __getattr__(self, item):
        return self._settings[item]



from enum import Enum
import datetime

import pytz


class TabStatus(Enum):
    AVAIL = "AVAILABLE"
    CLOSED = "CLOSED"
    NOT_YET = "NOT_YET"
    UNKNOWN = "UNKNOWN"
    ORDERED = "ORDERED"

    @classmethod
    def parse(cls, string_value):
        """
        :type string_value: str | unicode
        :rtype: TabStatus
        """
        value_enums = {
            "AVAILABLE": cls.AVAIL,
            "CLOSED": cls.CLOSED,
            "NOT_YET": cls.NOT_YET,
            "ORDER": cls.ORDERED,
        }
        return value_enums.get(string_value, cls.UNKNOWN)


class ReadableObject(object):
    def __repr__(self):
        return str(self)


class Address(ReadableObject):
    """ 地址 """

    def __init__(self, data):
        """
        :type data: dict
        """
        self.uid = data["uniqueId"]
        self.address = data["address"]  # 公司地址
        self.pick_up = data["pickUpLocation"]  # 取餐地址

    def __repr__(self):
        return "{} {}".format(self.uid, self.address)


class Tab(ReadableObject):
    """
    一个 Tab 代表一个时间窗口
    比如 中饭点餐时间
    或者 晚饭点餐时间
    """

    def __init__(self, data):
        """
        :type data: dict
        """
        self.title = data["title"]
        self.target_time = datetime.datetime.fromtimestamp(
            int(data["targetTime"]) / 1000, tz=pytz.timezone("Asia/Shanghai")
        )
        self.status = TabStatus.parse(data["status"])
        self.uid = data["userTab"]["uniqueId"]
        self.addresses = [Address(_) for _ in data["userTab"]["corp"]["addressList"]]

    def __repr__(self):
        return "{} {} {}".format(self.status.value, self.target_time.strftime("%Y-%m-%d"), self.title)


class Restaurant(ReadableObject):
    """ 餐厅 """

    def __init__(self, tab, data):
        """
        :type tab: Tab
        :type data: dict
        """
        self.tab = tab
        self.uid = data["uniqueId"]
        self.name = data["name"]
        self.is_open = data["open"]
        self.rating = data["rating"]
        self.tel = data["tel"]
        self.latitude = data["latitude"]
        self.longitude = data["longitude"]

    def __repr__(self):
        return "{}".format(self.name)


class Dish(ReadableObject):
    """ 菜 """

    def __init__(self, restaurant, data):
        """
        :type restaurant: Restaurant
        :type data: dict
        """
        self.restaurant = restaurant
        self.id = int(data["id"])
        self.name = data["name"]
        self.price = data["priceString"]

    def __repr__(self):
        return "{}".format(self.name)



import argparse
import sys

from meican.exceptions import NoOrderAvailable
from meican.settings import MeiCanSetting
from meican.tools import MeiCan


def initialize_meican():
    settings = MeiCanSetting()
    settings.load_credentials()
    return MeiCan(settings.username, settings.password)


def execute(argv=None):
    if argv is None:
        argv = sys.argv[1:] or []
    parser = argparse.ArgumentParser(description="命令行点美餐的工具")
    parser.add_argument("-o", "--order", help="order meal")
    args = parser.parse_args(argv)

    meican = initialize_meican()

    try:
        dishes = meican.list_dishes()
    except NoOrderAvailable:
        print("别急，下一顿还没开放订餐")
        return
    if args.order:
        keyword = args.order.decode("utf-8")
        dishes = [_ for _ in dishes if keyword in _.name]
        if len(dishes) == 1:
            meican.order(dishes[0])
            print("done!")
        elif not dishes:
            print("没有找到 {} 的对应菜品".format(keyword))
        else:
            print("找到多于一个菜品，请指定更详细的关键词")
            print("\n".join(["{}".format(_) for _ in dishes]))


if __name__ == "__main__":
    execute()



import datetime
import json
import time
from urllib.parse import urlencode

import requests

from meican.commands import get_dishes, get_restaurants, get_tabs
from meican.exceptions import MeiCanError, MeiCanLoginFail, NoOrderAvailable
from meican.models import TabStatus


class RestUrl(object):
    """ 用来存储 MeiCan Rest 接口的类 """

    @classmethod
    def get_base_url(cls, path, params=None, wrap=True):
        """
        :type path: str | unicode
        :type params: dict
        :type wrap: bool
        :rtype: str | unicode
        """
        if params:
            if wrap:
                params["noHttpGetCache"] = int(time.time() * 1000)
            path = "{}?{}".format(path, urlencode(sorted(params.items())))
        return "https://meican.com/{}".format(path)

    @classmethod
    def login(cls):
        return cls.get_base_url("account/directlogin")

    @classmethod
    def calender_items(cls, detail=False):
        today = datetime.date.today()
        one_week = datetime.timedelta(weeks=1)
        data = {
            "beginDate": today.strftime("%Y-%m-%d"),
            "endDate": (today + one_week).strftime("%Y-%m-%d"),
            "withOrderDetail": detail,
        }
        return cls.get_base_url("preorder/api/v2.1/calendarItems/list", data)

    @classmethod
    def restaurants(cls, tab):
        """
        :type tab: meican.models.Tab
        """
        data = {"tabUniqueId": tab.uid, "targetTime": tab.target_time}
        return cls.get_base_url("preorder/api/v2.1/restaurants/list", data)

    @classmethod
    def dishes(cls, restaurant):
        """
        :type restaurant: meican.models.Restaurant
        """
        tab = restaurant.tab
        data = {"restaurantUniqueId": restaurant.uid, "tabUniqueId": tab.uid, "targetTime": tab.target_time}
        return cls.get_base_url("preorder/api/v2.1/restaurants/show", data)

    @classmethod
    def order(cls, dish, address_uid=""):
        """
        :type dish: meican.models.Dish
        :type address_uid: str
        """
        tab = dish.restaurant.tab
        address_uid = address_uid or tab.addresses[0].uid
        data = {
            "order": json.dumps([{"count": "1", "dishId": "{}".format(dish.id)}]),
            "tabUniqueId": tab.uid,
            "targetTime": tab.target_time,
            "corpAddressUniqueId": address_uid,
            "userAddressUniqueId": address_uid,
        }
        return cls.get_base_url("preorder/api/v2.1/orders/add", data, wrap=False)


class MeiCan(object):
    def __init__(self, username, password, user_agent=None):
        """
        :type username: str | unicode
        :type password: str | unicode
        """
        self.responses = []
        self._session = requests.Session()
        user_agent = user_agent or "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:47.0) Gecko/20100101 Firefox/47.0"
        self._session.headers["User-Agent"] = user_agent
        self._calendar_items = None
        self._tabs = None

        form_data = {"username": username, "password": password, "loginType": "username", "remember": True}
        response = self._request("post", RestUrl.login(), form_data)
        if 200 != response.status_code or "用户名或密码错误" in response.text:
            raise MeiCanLoginFail("login fail because username or password incorrect")

    @property
    def tabs(self):
        """
        :rtype: list[meican.models.Tab]
        """
        if not self._tabs:
            self.load_tabs()
        return self._tabs

    @property
    def next_available_tab(self):
        """
        :rtype: meican.models.Tab
        """
        available_tabs = [_ for _ in self.tabs if _.status == TabStatus.AVAIL]
        return available_tabs[0] if available_tabs else None

    def load_tabs(self, refresh=False):
        if not self._calendar_items or refresh:
            self._calendar_items = self.http_get(RestUrl.calender_items())
            self._tabs = get_tabs(self._calendar_items)

    def get_restaurants(self, tab):
        """
        :type tab: meican.models.Tab
        :rtype: list[meican.models.Restaurant]
        """
        data = self.http_get(RestUrl.restaurants(tab))
        return get_restaurants(tab, data)

    def get_dishes(self, restaurant):
        """
        :type restaurant: meican.models.Restaurant
        """
        data = self.http_get(RestUrl.dishes(restaurant))
        return get_dishes(restaurant, data)

    def list_dishes(self, tab=None):
        """
        :type tab: meican.models.Tab
        :rtype: list[meican.models.Dish]
        """
        tab = tab or self.next_available_tab
        if not tab:
            raise NoOrderAvailable("Currently no available orders")
        restaurants = self.get_restaurants(tab)
        dishes = []
        for restaurant in restaurants:
            dishes.extend(self.get_dishes(restaurant))
        return dishes

    def order(self, dish, address_uid=""):
        """
        :type dish: meican.models.Dish
        :type address_uid: str
        """
        data = self.http_post(RestUrl.order(dish, address_uid=address_uid))
        return data

    def http_get(self, url, **kwargs):
        """
        :type url: str | unicode
        :rtype: dict | str | unicode
        """
        response = self._request("get", url, **kwargs)
        return response.json()

    def http_post(self, url, data=None, **kwargs):
        """
        :type url: str | unicode
        :type data: dict
        :rtype: dict | str | unicode
        """
        response = self._request("post", url, data, **kwargs)
        return response.json()

    def _request(self, method, url, data=None, **kwargs):
        """
        :type method: str | unicode
        :type url: str | unicode
        :type data: dict
        :type kwargs: dict
        :rtype: requests.Response
        """
        func = getattr(self._session, method)
        response = func(url, data=data, **kwargs)  # type: requests.Response
        response.encoding = response.encoding or "utf-8"
        self.responses.append(response)
        if response.status_code != 200:
            error = response.json()
            raise MeiCanError("[{}] {}".format(error.get("error", ""), error.get("error_description", "")))
        return response



class MeiCanError(Exception):
    """ MeiCan 包产生的错误 """


class MeiCanKeyError(MeiCanError):
    """ MeiCan 接口更改导致的 Dict Key 错误 """


class MeiCanLoginFail(MeiCanError):
    """ 用户名或密码错误，导致的登录失败 """


class NoOrderAvailable(MeiCanError):
    """ 目前还点不了餐 """
